//! Serial interface loopback
#![deny(unsafe_code)]
//#![deny(warnings)]
#![feature(proc_macro)]
#![no_std]

extern crate cortex_m_rtfm as rtfm;
extern crate f4;
extern crate heapless;

#[macro_use(block)]
extern crate nb;

#[macro_use]
extern crate cortex_m_debug;

use f4::prelude::*;
use f4::Serial;
use f4::time::Hertz;
use heapless::Vec;
use rtfm::app;

// CONFIGURATION
const BAUD_RATE: Hertz = Hertz(115_200);

// RTFM FRAMEWORK
app! {
    device: f4::stm32f40x,
}

// Init executes with interrupts disabled
// Hence its safe to access all peripherals (no race-conditions)
//
// In this case init will never return
// This is not the typical use-case as we will see later
fn init(p: init::Peripherals) {
    ipln!("init");
    let serial = Serial(p.USART2);

    serial.init(BAUD_RATE.invert(), None, p.GPIOA, p.RCC);

    let mut vector: Vec<u8, [u8; 4]> = Vec::new();
    loop {
        match block!(serial.read()) {
            Ok(byte) => {
                let _ = vector.push(byte);
                ipln!("Ok {:?}", vector);
                let _ = serial.write(byte);
            }
            Err(err) => {
                ipln!("Error {:?}: the bufer overflows. Type slower", err);
                p.USART2.dr.read(); // clear the error by reading the data register
            }
        }
    }
}

// We will never reach `idle` since we burn the CPU arduino style :)
fn idle() -> ! {
    // Sleep
    loop {
        rtfm::wfi();
    }
}

// 1. compile and run the project at 16MHz
// make sure its running (not paused)
// start a terminal program, e.g., `moserial`
// connect to the port
//
// Device       /dev/ttyACM0
// Baude Rate   115200
// Data Bits    8
// Stop Bits    1
// Parity       None
// Handshake    None
//
// (this is also known in short as 15200 8N1)
//
// you should now be able to send data and recive an echo from the MCU
//
// try sending: "abcd" as a single sequence (set the option No end in moserial)
// (don't send the quation marks, just abcd)
//
// what did you receive, and what was the output of the ITM trace
// ** your answer here **
//
// now try sending 'a', 'b', 'c', 'd' character by character
// (just send the characters not the single quotes and commas)
// what did you receive, and what was the output of the ITM trace
// ** your answer here **
//  a b c d it echo emidtly. //
// why did the transmission fail? (hint, think about timing...)
// ** your answer here **
//  it is set to 8 byte som it sent 1 charakter att the time and ther for is the echo per charakter
//
// commit your answers (bare7_1)
//
// 2. now stress the buffer lengt sending a sequence
// 'a', 'b', 'c', 'd', 'e' character by character
// what did you receive, and what was the output of the ITM trace
// ** your answer here **
//  "ok [96,97,98,99]
//  repit ...
//  
// if done correctly you see an evedince of Rust's memory safety
// the buffer will be saturated (all elements occupied)
// but no buffer owerwrite will occur (outside the buffer)
//
// your job now is to check the API of `heapless`
// https://docs.rs/heapless/0.2.1/heapless/
//
// and catch the case we are trying to write to a full buffer/vector
// and write a suiteble error message
//
// commit your answers (bare7_2)
//
// !!!!! NOTICE !!!!!
// here we are not solving the underlying problem
// we are just mitigating the effects
// in bare8 we will se how to use interrupts for reliable
// high-speed communictation
